# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import CreateableAPIResource, UpdatableAPIResource


class Comment(CreateableAPIResource, UpdatableAPIResource):
    """
    A wrapper around Comment HTTP API.
    """

    _resource_name = "comments"
